#!/usr/bin/python
#############################################################################
# program:	avrdump.py
# author:	Werner Hoch <werner.ho@gmx.de>
# license:	public domain without any warranty
# decription:	the program reads the binary data from the serial port
#		the file is converted to html and the data is assigned to 
#		the Port descriptions
#		data order is:
#		Offset: registers and rams that have to be rescued before
#			reading the ram (and the ram mapped ports and registers)
#			with the Offset data I can recover all register, port,..
#		RAM: in reverse order
#		PORTS: in reverse order
#		EEPROM: in reverse order
#############################################################################

import string, sys, cgi

#############################################################################
## constants
#INFILE='/dev/ttyS1'		# used for cgi script
INFILE='/dev/stdin'
OUTFILE='/dev/stdout'
OFFSET=5
RAM=128
PORTS=64
REGISTERS=32
EEPROM=128

sreg=0x5f
dumpreg=17
spl=0x5d
zl_reg=30
zh_reg=31

#############################################################################
## html constants
#cgi_head="Content-Type: text/html\n\n"		# used for cgi script
cgi_head=""
html_head='''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN">
<HTML>
<HEAD>
  <TITLE>AVR register-/port-/memory dump</TITLE>
</HEAD>
<BODY>
'''
html_table_start='''<TABLE width=100% BORDER=1>'''
html_tail='''</BODY></HTML>'''
#############################################################################
## variables
#         ADDR   DESC   BITNAMES                         COMMENT   VALUE
dumpmap={ 0x5F:("SREG",['I','T','H','S','V','N','Z','C'],"flag register",0x00),
          0x5D:("SPL",[],"low(stack pointer)",0x00),
          0x5B:("GIMSK",['INT1','INT0','','','','','',''],"interrupt mask",0x00),
          0x5A:("GIFR",['INTF1','INTF0','','','','','',''],"interrupt flags",0x00),
          0x59:("TIMSK",['TOIE1','OCIE1A','','','TICIE1','','TOIE0',''],"timer interupt mask",0x00),
          0x58:("TIFR",['TOV1','OCF1A','','','ICF1','','TOV0',''],"timer interrupt flags",0x00),
          0x55:("MCUCR",['','','SE','SM','ISC11','ISC10','ISC01','ISC00'],"MCU control",0x00),
          0x53:("TCCR0",['','','','','','CS02','CS01','CS00'],"timer prescale register",0x00),
          0x52:("TCNT0",[],"timer/counter0 (8bits)",0x00),
          0x4F:("TCCR1A",['COM1A1','COM1A0','','','','','PWM11','PWM10'],"timer/counter1 control register",0x00),
          0x4E:("TCCR1B",['ICNC1','ICES1','','','CTC1','CS12','CS11','CS10'],"timer/counter1 control register",0x00),
          0x4D:("TCNT1H",[],"timer/counter1- high(counter register)",0x00),
          0x4C:("TCNT1L",[],"timer/counter1- low(counter register)",0x00),
          0x4B:("OCR1AH",[],"timer/counter1- high(compare register)",0x00),
          0x4A:("OCR1AL",[],"timer/counter1- low(compare register)",0x00),
          0x45:("ICR1H",[],"timer/counter1- high(input capture register)",0x00),
          0x44:("ICR1L",[],"timer/counter1- low(input capture register",0x00),
          0x41:("WDTCR",['','','','WDTOE','WDE','WDP2','WDP1','WDP0'],"watchdog timer control",0x00),
          0x3E:("EEAR",[],"EEPROM adress register",0x00),
          0x3D:("EEDR",[],"EEPROM data register",0x00),
          0x3C:("EECR",['','','','','','EEMWE','EEWE','EERE'],"EEPROM control",0x00),
          0x38:("PORTB",[],"portb output/pullup",0x00),
          0x37:("DDRD",[],"port b direction register",0x00),
          0x36:("PINB",[],"port b input",0x00),
          0x32:("PORTD",[],"port d output/pullup",0x00),
          0x31:("DDRD",[],"port d direction register",0x00),
          0x30:("PIND",[],"port d input",0x00),
          0x2C:("UDR",[],"UART I/O data register",0x00),
          0x2B:("USR",['RXC','TXC','UDRE','FE','OR','','',''],"UART status register",0x00),
          0x2A:("UCR",['RXCIE','TXCIE','UDRIE','RXEN','TXEN','CHR9','RXB8','TXB8'],"UART control register",0x00),
          0x29:("UBRR",[],"UART baud rate register",0x00),
          0x28:("ACSR",['ACD','','AC0','ACI','ACIE','ACIC','ACIS1','ACIS0'],"analog comparator",0x00)
          }

#############################################################################
## functions
def tobin(n):
    hex=['0000','0001','0010','0011','0100','0101','0110','0111',
         '1000','1001','1010','1011','1100','1101','1110','1111']
    s=hex[n/16]+hex[n%16]+'b'
    return s

def tochar(n):
    if n < 128 and n > 31:
        letter = chr(n)
        if letter == "<":
            ret= "&lt;"
        elif letter == ">":
            ret = "&gt;"
        elif letter == "&":
            ret = "&amp;"
        else:
            ret = letter
        return chr(n)
    else:
        return "."

def num2str(n):
    s=tobin(n)
    c=tochar(n)
    return   '0x%02X , '%n + s + ' , %i'%n + ' , "' + c + '"'

def cgi_write():
    f_h.write(cgi_head)
    f_h.write(html_head)
    f_h.write('''<h3> register </h3>''')
    f_h.write(html_table_start)
    for i in range(REGISTERS/4):
        f_h.write('<tr>\n')
        for j in range(4):
            pos = addr = i + REGISTERS/4*j
            f_h.write('<td bgcolor="#FFAAAA">' +'r%i'%addr + '</td>')
            datum=ord(data_reg[pos])
            f_h.write('<td>' + num2str(datum) + '</td>')
        f_h.write('</tr>\n')
    f_h.write('</table> ')

    f_h.write('''<h3> ports </h3>''')
    f_h.write(html_table_start)
    for i in range(PORTS):
        addr = i + REGISTERS
        pos = i
        if dumpmap.has_key(addr):
            name, bits, descr, value = dumpmap[addr]
            f_h.write('<tr>\n')
            f_h.write('<td bgcolor="#AAFFAA">' +'0x%02X'%addr + '</td>')
            f_h.write('<td>' + name + '</td>')
            datum=ord(data_port[pos])
            s=tobin(datum)
            if len(bits) > 7 :
                for j in range(8):
                    if bits[j] != '':
                        f_h.write('<td>' + bits[j] +'='+ s[j] +'</td>')
                    else:
                        f_h.write('<td>' + '</td>')
            else:
                f_h.write('<td colspan="8" align="center">'
                          + num2str(datum) + '</td>')
            f_h.write('<td>' + descr + '</td>')
            f_h.write('</tr>\n')
    f_h.write('</table>')

    f_h.write('''<h3> memory </h3>''')
    f_h.write(html_table_start)
    stackpointer=ord(data_port[spl-REGISTERS])
    for i in range(RAM/4):
        f_h.write('<tr>\n')
        for j in range(4):
            pos = i + RAM/4*j
            addr = REGISTERS + PORTS + pos
            color='<td bgcolor="#AAAAFF">'
            if  addr == stackpointer :
                color = '<td bgcolor="#FFAAAA">'
            f_h.write(color +'0x%02X'%addr + '</td>')
            datum=ord(data_ram[pos])
            f_h.write('<td>' + num2str(datum) + '</td>')
        f_h.write('</tr>\n')
    f_h.write('</table> ')

    f_h.write('''<h3> eeprom </h3>''')
    f_h.write(html_table_start)
    for i in range(EEPROM/4):
        f_h.write('<tr>\n')
        for j in range(4):
            pos = addr = i + EEPROM/4*j
            f_h.write('<td bgcolor="#FFFFAA">' +'0x%02x'%addr + '</td>')
            datum=ord(data_eepr[pos])
            f_h.write('<td>' + num2str(datum) + '</td>')
        f_h.write('</tr>\n')
    f_h.write('</table> ')
    f_h.write(html_tail)

#############################################################################
## MAIN

f=open(INFILE,"rb")
data=f.read(EEPROM+OFFSET+RAM+REGISTERS+PORTS)
f.close()
f_h=open(OUTFILE,"wt")

## data order is: OFFSET, RAM, PORTS, REGS, EEPROM
data_ram=[]
data_port=[]
data_reg=[]
data_eepr=[]
l=OFFSET+RAM
for i in range(RAM):
    data_ram.append(data[l-i-1])

l=OFFSET+RAM+PORTS
for i in range(PORTS):
    data_port.append(data[l-i-1])

l=OFFSET+RAM+PORTS+REGISTERS
for i in range(REGISTERS):
    data_reg.append(data[l-i-1])

l=EEPROM+OFFSET+RAM+REGISTERS+PORTS
for i in range(EEPROM):
    n = l-i-1
    data_eepr.append(data[l-i-1])
    
## get ZH and ZL from the stack
stackpointer=ord(data_port[spl-REGISTERS])
data_reg[zl_reg]=data_ram[stackpointer-REGISTERS-PORTS+2]
data_reg[zh_reg]=data_ram[stackpointer-REGISTERS-PORTS+1]
## correct the bytes inside the offset
data_reg[dumpreg]=data[0]
data_port[sreg- REGISTERS]=data[1]
data_port[spl- REGISTERS]=data[2]
data_ram[ord(data_port[spl-REGISTERS])-1-PORTS-REGISTERS]=data[3]
data_ram[ord(data_port[spl-REGISTERS])-0-PORTS-REGISTERS]=data[4]

## html output    
cgi_write()
